#
# 2D curve demo applet
#
# doug zongker (dougz@cs.washington.edu)
# 17 november 1999
#

if {[catch {load debug/curvedemo}]} {
    if {[catch {load curvedemo}]} {
	error "can't find curvedemo.dll!"
    }
}

set g_points  {}
set g_tension 0.5
set g_control 1
set g_multiplicity 1
set g_alternate 0
set g_type "bspline"

canvas .c -width 800 -height 600
.c create rect -10 -10 2000 2000 -fill [.c cget -bg] -tags bg
pack .c -expand 1 -fill both
.c bind bg <1> [list create_point %x %y]

frame .b
tk_optionMenu .b.types g_type {cubic beziers} bspline catmullrom {c2 interpolating}
button .b.clear -text "clear" -command clear_points
button .b.quit -text "quit" -command exit
pack .b.types .b.clear .b.quit -side left

frame .b2
radiobutton .b2.single -text "single" -variable g_multiplicity -value 1 -command redraw
radiobutton .b2.double -text "double" -variable g_multiplicity -value 2 -command redraw
radiobutton .b2.triple -text "triple" -variable g_multiplicity -value 3 -command redraw
checkbutton .b2.control -text "control polygon" -variable g_control -command redraw
checkbutton .b2.alternate -text "alternate" -variable g_alternate -command redraw
label .b2.l -text "tension:"
scale .b2.tension -from 0.0 -to 2.0 -res 0.1 -variable g_tension -command redraw -orient h -state disabled
pack .b2.single .b2.double .b2.triple .b2.control .b2.alternate .b2.l -side left
pack .b2.tension -expand 1 -fill x -side left

pack .b .b2 -fill x

trace variable g_type w redraw

proc redraw {{args}} {
    global g_points g_endpoint g_type g_types g_control g_alternate g_multiplicity g_tension
    set colors [list red yellow]
    set color 0
    set dotsize 6
    .c delete stuff

    if {$g_type == "bspline"} {
	.b2.single config -state normal
	.b2.double config -state normal
	.b2.triple config -state normal
    } else {
	.b2.single config -state disabled
	.b2.double config -state disabled
	.b2.triple config -state disabled
    } 

    if {$g_type == "catmullrom"} {
	.b2.tension config -state normal
    } else {
	.b2.tension config -state disabled
    }
    
    if {$g_control} {
	# draw control polygon
	set oldp [lindex $g_points 0]
	foreach p [lrange $g_points 1 end] {
	    .c create line [lindex $oldp 0] [lindex $oldp 1] [lindex $p 0] [lindex $p 1] -width 3 -fill "#a0a0a0" -tags stuff
	    set oldp $p
	}
    }

    switch -- $g_type {
	"cubic beziers" {
	    set curve [curvedemo::bezier3 $g_points]
	}
	"bspline" {
	    set curve [curvedemo::bspline $g_multiplicity $g_points]
	}
	"catmullrom" {
	    set curve [curvedemo::catmullrom $g_tension $g_points]
	}
	"c2 interpolating" {
	    set curve [curvedemo::c2interpolating $g_points]
	}
	default {
	    set curve {}
	}
    }
    
    if {[llength $curve] > 1} {
	if {$g_alternate} {
	    set div [curvedemo::getdivision]
	    set segs [expr [llength $curve]/(2*$div)]
	    if {$g_type == "bspline" && $g_multiplicity == 2} {
		set color "yellow"
	    } else {
		set color "red"
	    }
	    for {set i 0} {$i < $segs} {incr i} {
		eval .c create line [lrange $curve [expr $i*2*$div] [expr ($i+1)*2*$div+1]] -fill $color -width 2 -tags stuff
		if {$color == "red"} {
		    set color "yellow"
		} else {
		    set color "red"
		}
	    }
	} else {
	    eval .c create line $curve -fill red -width 2 -tags stuff
	}
    }
    
    # draw control points
    set i 0
    foreach p $g_points {
	set x [lindex $p 0]
	set y [lindex $p 1]
	set id [.c create oval [expr $x-$dotsize] [expr $y-$dotsize] [expr $x+$dotsize] [expr $y+$dotsize] -fill blue -tags stuff -outline {}]
	.c bind $id <ButtonPress-1> "point_down $i %x %y"
	incr i
    }

}

proc clear_points {} {
    global g_points 
    
    .c delete stuff
    set g_points {}
}

proc point_down {num x y} {
    global g_points
    bind .c <Motion> "point_move $num %x %y"
    bind .c <ButtonRelease-1> "point_up"
}

proc point_move {num x y} {
    global g_points
    set g_points [concat [lrange $g_points 0 [expr $num-1]] \
	    [list [list [.c canvasx $x] [.c canvasy $y]]] \
	    [lrange $g_points [expr $num+1] end]]
    redraw
}

proc point_up {} {
    bind .c <Motion> ""
    bind .c <ButtonRelease-3> ""
}

proc create_point {x y} {
    global g_points
    
    set x [.c canvasx $x]
    set y [.c canvasy $y]

    lappend g_points [list $x $y]
    redraw
}


    

    
    
    
